@@ -382,6 +382,10 @@ g_vfs_ftp_task_set_error_from_response (GVfsFtpTask *task, guint response)
         code = G_IO_ERROR_NOT_SUPPORTED;
         msg = _("Operation unsupported");
         break;
+      case 522: /* EPRT: unsupported network protocol */
+        code = G_IO_ERROR_NOT_SUPPORTED;
+        msg = _("Unsupported network protocol");
+        break;
       case 530: /* Not logged in. */
         code = G_IO_ERROR_PERMISSION_DENIED;
         msg = _("Permission denied");
@@ -747,7 +751,7 @@ g_vfs_ftp_task_create_remote_address (GVfsFtpTask *task, guint port)
 }
 
 static GVfsFtpMethod
-g_vfs_ftp_task_open_data_connection_epsv (GVfsFtpTask *task, GVfsFtpMethod method)
+g_vfs_ftp_task_setup_data_connection_epsv (GVfsFtpTask *task, GVfsFtpMethod method)
 {
   const char *s;
   char **reply;
@@ -790,7 +794,7 @@ fail:
 }
 
 static GVfsFtpMethod
-g_vfs_ftp_task_open_data_connection_pasv (GVfsFtpTask *task, GVfsFtpMethod method)
+g_vfs_ftp_task_setup_data_connection_pasv (GVfsFtpTask *task, GVfsFtpMethod method)
 {
   guint ip1, ip2, ip3, ip4, port1, port2;
   char **reply;
@@ -799,7 +803,6 @@ g_vfs_ftp_task_open_data_connection_pasv (GVfsFtpTask *task, GVfsFtpMethod metho
   guint status;
   gboolean success;
 
-  /* only binary transfers please */
   status = g_vfs_ftp_task_send_and_check (task, 0, NULL, NULL, &reply, "PASV");
   if (status == 0)
     return G_VFS_FTP_METHOD_ANY;
@@ -872,17 +875,103 @@ g_vfs_ftp_task_open_data_connection_pasv (GVfsFtpTask *task, GVfsFtpMethod metho
   return G_VFS_FTP_METHOD_ANY;
 }
 
+static GVfsFtpMethod
+g_vfs_ftp_task_setup_data_connection_eprt (GVfsFtpTask *task, GVfsFtpMethod unused)
+{
+  GSocketAddress *addr;
+  guint status, port, family;
+  char *ip_string;
+
+  /* workaround for the task not having a connection yet */
+  if (task->conn == NULL &&
+      g_vfs_ftp_task_send (task, 0, "NOOP") == 0)
+    return G_VFS_FTP_METHOD_ANY;
+
+  addr = g_vfs_ftp_connection_listen_data_connection (task->conn, &task->error);
+  if (addr == NULL)
+    return G_VFS_FTP_METHOD_ANY;
+  switch (g_socket_address_get_family (addr))
+    {
+      case G_SOCKET_FAMILY_IPV4:
+        family = 1;
+        break;
+      case G_SOCKET_FAMILY_IPV6:
+        family = 2;
+        break;
+      default:
+        g_object_unref (addr);
+        return G_VFS_FTP_METHOD_ANY;
+    }
+
+  ip_string = g_inet_address_to_string (g_inet_socket_address_get_address (G_INET_SOCKET_ADDRESS (addr)));
+  /* if this ever happens (and it must not for IP4 and IP6 addresses), 
+   * we need to add support for using a different separator */
+  g_assert (strchr (ip_string, '|') == NULL);
+  port = g_inet_socket_address_get_port (G_INET_SOCKET_ADDRESS (addr));
+
+  /* we could handle the 522 response here, (unsupported network family),
+   * but I don't think that will buy us anything */
+  status = g_vfs_ftp_task_send (task, 0, "EPRT |%u|%s|%u|", family, ip_string, port);
+  g_free (ip_string);
+  g_object_unref (addr);
+  if (status == 0)
+    return G_VFS_FTP_METHOD_ANY;
+  
+  return G_VFS_FTP_METHOD_EPRT;
+}
+
+static GVfsFtpMethod
+g_vfs_ftp_task_setup_data_connection_port (GVfsFtpTask *task, GVfsFtpMethod unused)
+{
+  GSocketAddress *addr;
+  guint status, i, port;
+  char *ip_string;
+
+  /* workaround for the task not having a connection yet */
+  if (task->conn == NULL &&
+      g_vfs_ftp_task_send (task, 0, "NOOP") == 0)
+    return G_VFS_FTP_METHOD_ANY;
+
+  addr = g_vfs_ftp_connection_listen_data_connection (task->conn, &task->error);
+  if (addr == NULL)
+    return G_VFS_FTP_METHOD_ANY;
+  /* the PORT command only supports IPv4 */
+  if (g_socket_address_get_family (addr) != G_SOCKET_FAMILY_IPV4)
+    {
+      g_object_unref (addr);
+      return G_VFS_FTP_METHOD_ANY;
+    }
+
+  ip_string = g_inet_address_to_string (g_inet_socket_address_get_address (G_INET_SOCKET_ADDRESS (addr)));
+  for (i = 0; ip_string[i]; i++)
+    {
+      if (ip_string[i] == '.')
+        ip_string[i] = ',';
+    }
+  port = g_inet_socket_address_get_port (G_INET_SOCKET_ADDRESS (addr));
+
+  status = g_vfs_ftp_task_send (task, 0, "PORT %s,%u,%u", ip_string, port >> 8, port & 0xFF);
+  g_free (ip_string);
+  g_object_unref (addr);
+  if (status == 0)
+    return G_VFS_FTP_METHOD_ANY;
+  
+  return G_VFS_FTP_METHOD_PORT;
+}
+
 typedef GVfsFtpMethod (* GVfsFtpOpenDataConnectionFunc) (GVfsFtpTask *task, GVfsFtpMethod method);
 
 static GVfsFtpMethod
-g_vfs_ftp_task_open_data_connection_any (GVfsFtpTask *task, GVfsFtpMethod unused)
+g_vfs_ftp_task_setup_data_connection_any (GVfsFtpTask *task, GVfsFtpMethod unused)
 {
   static const struct {
     GVfsFtpFeature required_feature;
     GVfsFtpOpenDataConnectionFunc func;
   } funcs_ordered[] = {
-    { G_VFS_FTP_FEATURE_EPSV, g_vfs_ftp_task_open_data_connection_epsv },
-    { 0,                      g_vfs_ftp_task_open_data_connection_pasv }
+    { G_VFS_FTP_FEATURE_EPSV, g_vfs_ftp_task_setup_data_connection_epsv },
+    { 0,                      g_vfs_ftp_task_setup_data_connection_pasv },
+    { G_VFS_FTP_FEATURE_EPRT, g_vfs_ftp_task_setup_data_connection_eprt },
+    { 0,                      g_vfs_ftp_task_setup_data_connection_port }
   };
   GVfsFtpMethod method;
   guint i;
@@ -918,30 +1007,33 @@ g_vfs_ftp_task_open_data_connection_any (GVfsFtpTask *task, GVfsFtpMethod unused
 }
 
 /**
- * g_vfs_ftp_task_open_data_connection:
+ * g_vfs_ftp_task_setup_data_connection:
  * @task: a task not having an open data connection
  *
- * Tries to open a data connection to the ftp server. If the operation fails,
- * @task will be set into an error state.
+ * Sets up a data connection to the ftp server with using the best method for 
+ * this task. If the operation fails, @task will be set into an error state.
+ * You must call g_vfs_ftp_task_open_data_connection() to finish setup and 
+ * ensure the data connection actually gets opened. Usually, this requires 
+ * sending an FTP command down the stream.
  **/
 void
-g_vfs_ftp_task_open_data_connection (GVfsFtpTask *task)
+g_vfs_ftp_task_setup_data_connection (GVfsFtpTask *task)
 {
   static const GVfsFtpOpenDataConnectionFunc connect_funcs[] = {
-    [G_VFS_FTP_METHOD_ANY] = g_vfs_ftp_task_open_data_connection_any,
-    [G_VFS_FTP_METHOD_EPSV] = g_vfs_ftp_task_open_data_connection_epsv,
-    [G_VFS_FTP_METHOD_PASV] = g_vfs_ftp_task_open_data_connection_pasv,
-    [G_VFS_FTP_METHOD_PASV_ADDR] = g_vfs_ftp_task_open_data_connection_pasv,
-    [G_VFS_FTP_METHOD_EPRT] = NULL,
-    [G_VFS_FTP_METHOD_PORT] = NULL
+    [G_VFS_FTP_METHOD_ANY]       = g_vfs_ftp_task_setup_data_connection_any,
+    [G_VFS_FTP_METHOD_EPSV]      = g_vfs_ftp_task_setup_data_connection_epsv,
+    [G_VFS_FTP_METHOD_PASV]      = g_vfs_ftp_task_setup_data_connection_pasv,
+    [G_VFS_FTP_METHOD_PASV_ADDR] = g_vfs_ftp_task_setup_data_connection_pasv,
+    [G_VFS_FTP_METHOD_EPRT]      = g_vfs_ftp_task_setup_data_connection_eprt,
+    [G_VFS_FTP_METHOD_PORT]      = g_vfs_ftp_task_setup_data_connection_port
   };
   GVfsFtpMethod method, result;
 
   g_return_if_fail (task != NULL);
 
-  /* FIXME: get the method from elsewhere */
+  task->method = G_VFS_FTP_METHOD_ANY;
+
   method = g_atomic_int_get (&task->backend->method);
-  
   g_assert (method < G_N_ELEMENTS (connect_funcs) && connect_funcs[method]);
 
   if (g_vfs_ftp_task_is_in_error (task))
@@ -953,7 +1045,7 @@ g_vfs_ftp_task_open_data_connection (GVfsFtpTask *task)
   if (result == G_VFS_FTP_METHOD_ANY &&
       method != G_VFS_FTP_METHOD_ANY &&
       !g_vfs_ftp_task_is_in_error (task))
-    result = g_vfs_ftp_task_open_data_connection_any (task, G_VFS_FTP_METHOD_ANY);
+    result = g_vfs_ftp_task_setup_data_connection_any (task, G_VFS_FTP_METHOD_ANY);
 
   g_assert (result < G_N_ELEMENTS (connect_funcs) && connect_funcs[result]);
   if (result != method)
@@ -970,5 +1062,28 @@ g_vfs_ftp_task_open_data_connection (GVfsFtpTask *task)
       g_debug ("# set default data connection method from %s to %s\n",
                methods[method], methods[result]);
     }
+  task->method = result;
+}
+
+/**
+ * g_vfs_ftp_task_open_data_connection:
+ * @task: a task
+ *
+ * Tries to open a data connection to the ftp server. If the operation fails,
+ * @task will be set into an error state.
+ **/
+void
+g_vfs_ftp_task_open_data_connection (GVfsFtpTask *task)
+{
+  g_return_if_fail (task != NULL);
+
+  if (g_vfs_ftp_task_is_in_error (task))
+    return;
+
+  if (task->method == G_VFS_FTP_METHOD_EPRT ||
+      task->method == G_VFS_FTP_METHOD_PORT)
+    g_vfs_ftp_connection_accept_data_connection (task->conn, 
+                                                 task->cancellable,
+                                                 &task->error);
 }
 
